@@ -18,8 +18,6 @@
 
 #include <config.h>
 
-#include <gnome.h>
-
 #include "gconf-editor-window.h"
 
 #include "gconf-bookmarks.h"
@@ -29,13 +27,17 @@
 #include "gconf-cell-renderer.h"
 #include "gconf-editor-application.h"
 #include "gconf-key-editor.h"
+#include "gconf-policykit.h"
 #include "gconf-stock-icons.h"
 #include "gconf-util.h"
 #include "gedit-output-window.h"
 #include "gconf-search-dialog.h"
-#include <gconf/gconf.h>
-#include <string.h>
 
+#include <gconf/gconf.h>
+#include <glib/gi18n.h>
+#include <gdk/gdkx.h>
+#include <string.h>
+#include <stdlib.h>
 
 /* Property IDs */
 enum
@@ -94,21 +96,8 @@ gconf_editor_window_go_to (GConfEditorWindow *window,
 	gint depth;
 	gint i, nchildren;
 	char *key;
-	gboolean dir_exists;
-	GError *error = NULL;
 	
 	g_return_if_fail (location != NULL);
-		
-	dir_exists = gconf_client_dir_exists (window->client, location, &error);
-	
-	if (error != NULL) {
-    		gconf_editor_window_popup_error_dialog (GTK_WINDOW (window),"%s", error);
-        	return;
-	}
-	
-	if (!gconf_client_get (window->client, location, NULL) && !dir_exists)
-		return;
-	
 	
 	child_path = gconf_tree_model_get_tree_path_from_gconf_path (
 				GCONF_TREE_MODEL (window->tree_model), location);
@@ -256,6 +245,7 @@ gconf_add_recent_key (GConfEditorWindow *window, const char *key)
 		/* Keep list on a fixed size, remove the last */
 		if (g_slist_length (list) > RECENT_LIST_MAX_SIZE - 1) {
 			tmp = g_slist_last (list);
+			g_free (tmp->data);
 			list = g_slist_delete_link (list, tmp);
 		}
 
@@ -360,8 +350,9 @@ static void
 help_cb (GtkAction *action, GtkWidget *callback_data)
 {
         GError *error = NULL;
-                                                                                
-        gnome_help_display_desktop (NULL, "gconf-editor", "gconf-editor", NULL, &error);
+
+	gtk_show_uri (gtk_widget_get_screen (callback_data),
+		      "ghelp:gconf-editor", gtk_get_current_event_time (), &error);
                                                                                 
         if (error != NULL) {
                 gconf_editor_window_popup_error_dialog (GTK_WINDOW (GCONF_EDITOR_WINDOW (callback_data)),
@@ -649,24 +640,25 @@ gconf_editor_window_row_expanded (GtkTreeView       *tree_view,
   gdk_display_flush (gtk_widget_get_display (GTK_WIDGET (gconfwindow)));
 }
 
+static void
+gconf_editor_popup_policykit_callback (GtkWindow *window, GError *error)
+{
+  if (error)
+    gconf_editor_window_popup_error_dialog (window, _("Could not set value. Error was:\n%s"), error);
+}
+
 static void
 gconf_editor_popup_window_set_as_default (GtkAction *action, GtkWidget *callback_data)
 {
   GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (callback_data);
-  GtkWindow *window = GTK_WINDOW (callback_data);
 
-  GConfClient *client;
-  GError *error = NULL;
   GtkTreeIter iter;
   GConfValue *value;
   char *path = NULL;
+  GtkWindow *window = GTK_WINDOW (callback_data);
+  gboolean can_use_pk;
   
-  client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
-  if (client == NULL) {
-    g_print ("Could not create GConf client\n");
-    return;
-  }
-	
+  can_use_pk = (gconfwindow->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL);
   gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (gconfwindow->list_view)),
                                    NULL, &iter);
   gtk_tree_model_get (gconfwindow->sorted_list_model, &iter,
@@ -674,34 +666,43 @@ gconf_editor_popup_window_set_as_default (GtkAction *action, GtkWidget *callback
                       GCONF_LIST_MODEL_VALUE_COLUMN, &value,
                       -1);
 
-  gconf_client_set (client, path, value, NULL);
+  if (gconf_util_can_edit_defaults ()) {
+    GConfClient *client;
 
-  gconf_client_suggest_sync (client, &error);
-  if (error) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
+    if (client != NULL) {
+      GError *error = NULL;
+
+      gconf_client_set (client, path, value, NULL);
+      gconf_client_suggest_sync (client, &error);
+      g_object_unref (client);
+
+      if (!error)
+        return;
+      if (!can_use_pk)
+        gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    }
+  }
+  if (can_use_pk) {
+    gconf_client_suggest_sync (gconfwindow->client, NULL);
+    gconf_pk_set_default_async (path,
+                                (GFunc) gconf_editor_popup_policykit_callback,
+                                g_object_ref (window), g_object_unref);
   }
-  g_object_unref (client);
 }
 
 static void
 gconf_editor_popup_window_set_as_mandatory (GtkAction *action, GtkWidget *callback_data)
 {
   GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (callback_data);
-  GtkWindow *window = GTK_WINDOW (callback_data);
 
-  GError *error = NULL;
   GtkTreeIter iter;
   GConfValue *value;
   char *path = NULL;
-  GConfClient *client;
+  GtkWindow *window = GTK_WINDOW (callback_data);
+  gboolean can_use_pk;
   
-
-  client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_MANDATORY);
-  if (client == NULL) {
-    g_print ("Could not create GConf client\n");
-    return;
-  }
-	
+  can_use_pk = (gconfwindow->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL);
   gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (gconfwindow->list_view)),
                                    NULL, &iter);
   gtk_tree_model_get (gconfwindow->sorted_list_model, &iter,
@@ -709,13 +710,29 @@ gconf_editor_popup_window_set_as_mandatory (GtkAction *action, GtkWidget *callba
                       GCONF_LIST_MODEL_VALUE_COLUMN, &value,
                       -1);
 
-  gconf_client_set (client, path, value, NULL);
+  if (gconf_util_can_edit_mandatory ()) {
+    GConfClient *client;
 
-  gconf_client_suggest_sync (client, &error);
-  if (error) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_MANDATORY);
+    if (client != NULL) {
+      GError *error = NULL;
+
+      gconf_client_set (client, path, value, NULL);
+      gconf_client_suggest_sync (client, &error);
+      g_object_unref (client);
+
+      if (!error)
+        return;
+      if (!can_use_pk)
+        gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    }
+  }
+  if (can_use_pk) {
+    gconf_client_suggest_sync (gconfwindow->client, NULL);
+    gconf_pk_set_mandatory_async (path,
+                                  (GFunc) gconf_editor_popup_policykit_callback,
+                                  g_object_ref (window), g_object_unref);
   }
-  g_object_unref (client);
 }
 
 static GtkActionEntry entries[] = {
@@ -906,11 +923,13 @@ list_view_button_press_event (GtkTreeView *tree_view, GdkEventButton *event, GCo
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/EditKey"), 
 						  TRUE);
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/UnsetKey"), 
-						  TRUE);
+						  window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/DefaultKey"), 
-						  TRUE);
+						  (gconf_util_can_edit_defaults () && window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS) ||
+						  (gconf_pk_can_set_default ("/") && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/MandatoryKey"), 
-						  TRUE);
+						  (gconf_util_can_edit_mandatory () && window->type != GCONF_EDITOR_WINDOW_TYPE_MANDATORY) ||
+						  (gconf_pk_can_set_mandatory ("/") && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 			
 			gtk_tree_path_free (path);
 		}
@@ -1003,16 +1022,17 @@ gconf_editor_window_list_view_popup_menu (GtkWidget *widget, GConfEditorWindow *
 
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->list_view));
 	/* Select our row */
-	if (gtk_tree_selection_get_selected (selection, &window->list_model, &iter)) {
+	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/EditKey"), 
 					  TRUE);
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/UnsetKey"), 
-					  TRUE);
+					  window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/DefaultKey"), 
-					  TRUE);
+					  (gconf_util_can_edit_defaults () && window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS) ||
+					  (gconf_pk_can_set_default ("/") && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/MandatoryKey"), 
-					  TRUE);
-			
+					  (gconf_util_can_edit_mandatory () && window->type != GCONF_EDITOR_WINDOW_TYPE_MANDATORY) ||
+					  (gconf_pk_can_set_mandatory ("/") && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 	}
 	else {
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/EditKey"), 
@@ -1070,7 +1090,7 @@ strip_whitespace (const char *text)
 		p = g_utf8_next_char (p);
 	}
 
-	return str->str;
+	return g_string_free (str, FALSE);
 }
 
 static void
@@ -1391,7 +1411,7 @@ gconf_editor_window_finalize (GObject *object)
 
 	g_object_unref (client);
 	g_object_unref (window->client);
-	
+
 	G_OBJECT_CLASS (gconf_editor_window_parent_class)->finalize (object);
     
 }
