@@ -18,21 +18,48 @@
 
 #include <config.h>
 
-#include <gnome.h>
+#include <stdlib.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf.h>
 
 #include "gconf-editor-application.h"
 #include "gconf-stock-icons.h"
 #include "gconf-editor-window.h"
 
+static char *
+build_accel_filename (void)
+{
+	return g_build_filename (g_get_home_dir (), ".gnome2", "accels", PACKAGE, NULL);
+}
+
+static void
+load_accel_map (void)
+{
+	char *map;
+
+	map = build_accel_filename ();
+	gtk_accel_map_load (map);
+
+	g_free (map);
+}
+
+static void
+save_accel_map (void)
+{
+	char *map;
+
+	map = build_accel_filename ();
+	gtk_accel_map_save (map);
+
+	g_free (map);
+}
 
 gint
 main (gint argc, gchar **argv)
 {
-	
-	GnomeProgram *program;
 	GOptionContext *context;
 	GtkWidget *window;
+	GError *error = NULL;
 
 	static gchar **remaining_args = NULL;
 	gchar *initial_key = NULL;
@@ -50,18 +77,22 @@ main (gint argc, gchar **argv)
 	context = g_option_context_new (N_("- Directly edit your entire configuration database"));
 
 	g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
-
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
-	program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
-									argc, argv,
-									GNOME_PARAM_GOPTION_CONTEXT, context,								
-									GNOME_PARAM_APP_DATADIR, DATADIR,
-									GNOME_PARAM_NONE);
+	if (!g_option_context_parse (context, &argc, &argv, &error)) {
+		g_critical ("Failed to parse arguments: %s", error->message);
+		g_error_free (error);
+		g_option_context_free (context);
+		exit (1);
+	}
+
+	g_option_context_free (context);
 
 	/* Register our stock icons */
         gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), ICONDIR);
 	gconf_stock_icons_register ();
+	load_accel_map ();
 
         gtk_window_set_default_icon_name ("gconf-editor");
 
@@ -76,10 +107,7 @@ main (gint argc, gchar **argv)
 	
 	gtk_main ();
 
-	gnome_accelerators_sync();
-	
-	g_object_unref(program);
-
+	save_accel_map ();
 	g_strfreev (remaining_args);
 
 	return 0;
